/***************************************************************************
           ulxr_connection.h  -  provide a connection for rpc-data
                             -------------------
    begin                : Sun Mar 24 2002
    copyright            : (C) 2002-2004 by Ewald Arnold
    email                : ulxmlrpcpp@ewald-arnold.de
 
    $Id: ulxr_connection.h,v 1.34 2004/08/03 21:01:22 ewald-arnold Exp $

 ***************************************************************************/

/**************************************************************************
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#ifndef ULXR_CONNECTION_H
#define ULXR_CONNECTION_H

#include <ulxmlrpcpp.h>  // always first header

#if defined(__BORLANDC__)  || defined (_MSC_VER)
typedef long ssize_t; // TODO detect
#endif

namespace ulxr {


/** Base class for connection between XML RPC client and server.
  */
class ULXR_EXPORT Connection
{
 public:

 /** Constructs a connection.
   * The connection is not yet open after construction.
   */
   Connection();

 /** Destroys the connection.
   * The connection is closed now at the latest.
   */
   virtual ~Connection();

 /** Closes the connection.
   */
   virtual void close();

 /** Detaches the connection by creating a duplicate of
   * the connection and closing the original connection afterwards.
   * @return the current connection
   */
   virtual Connection *detach() = 0;

 /** Writes data to the connection.
   * @param  buff pointer to data
   * @param  len  valid buffer length
   */
   virtual void write(char const *buff, long len);

 /** Reads data from the connection.
   * @param  buff pointer to data buffer
   * @param  len  maimum number of bytes to read into buffer
   * @return number of actually read bytes
   */
   virtual ssize_t read(char *buff, long len);

 /** Opens the connection in rpc client mode.
   */
   virtual void open() = 0;

 /** Opens the connection in rpc server mode, thus waiting for
   * connections from clients.
   */
   virtual void accept() = 0;

 /** Tests if the connection is open.
   * @return true if connection is already open.
   */
   virtual bool isOpen() const;

 /** Sets the maximum time it is waited to complete an action like open or read.
   * @param  to_sec  time in seconds
   */
   void setTimeout(unsigned to_sec)
     { timeoutmax = to_sec; }

  /** Portable function to return the current error number.
   * @return error number (errno under Unices)
   */
   virtual int getLastError();

  /** Portable function to return the error description for a given error number
   * @param  err_number    system dependent error code
   * @return error string
   */
   virtual CppString getErrorString(int err_number);

 /** Tests if the current transmission expects a return value.
   * @return true: return value for request
   */
   virtual CppString getInterfaceName() = 0;

 /** Cuts the connection.
   * Just the variables are reset, the connection itself is not touched.
   */
   virtual void cut();

 /** Returns the file handle of the connection.
   * @return  the file handle 
   */
   int getHandle() const;

 protected:
   
   int       fd_handle;
 
 private:
 
   unsigned  timeoutmax;
 
 /** Checks if there is input dta which can innediately be read.
   * @return true: data available
   */
   virtual bool hasPendingInput() const;
 
 /** Initializes internal variables.
   */
   void init();

 /** Actually writes data to the connection.
   * @param  buff pointer to data
   * @param  len  valid buffer length
   * @return  result from api write function
   */
   ssize_t virtual low_level_write(char const *buff, long len);

 /** Reads data from the connection.
   * @param  buff pointer to data buffer
   * @param  len  maimum number of bytes to read into buffer
   * @return  result from api read function
   */
   ssize_t virtual low_level_read(char *buff, long len);
   
};


};  // namespace ulxr


#endif // ULXR_CONNECTION_H
