/***************************************************************************
         ulxr_dispatcher.h  -  answer rpc requests ("rpc-server")
                             -------------------
    begin                : Sat Mar 23 2002
    copyright            : (C) 2002-2004 by Ewald Arnold
    email                : ulxmlrpcpp@ewald-arnold.de

    $Id: ulxr_dispatcher.h,v 1.43 2004/05/15 08:03:47 ewald-arnold Exp $

 ***************************************************************************/

/**************************************************************************
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#ifndef ULXR_DISPATCHER_H
#define ULXR_DISPATCHER_H

#include <ulxmlrpcpp.h>  // always first header
#include <ulxr_call.h>
#include <ulxr_response.h>


namespace ulxr {


class Protocol;
class Connection;
class Struct;


namespace hidden {


/** Internal helper class, not intended for public use.
 */
class ULXR_EXPORT MethodWrapperBase
{
  public:
    virtual MethodResponse call(const MethodCall &calldata) const = 0;
};


/** Internal helper class template, not intended for public use.
 */
template <class T>
class ULXR_EXPORT MethodWrapper : public MethodWrapperBase
{
  public:
   typedef MethodResponse (T::*PMF)(const MethodCall &calldata);

   virtual MethodResponse call (const MethodCall &calldata) const
   {
      return (obj->*adr) (calldata);
   }

   MethodWrapper(T *o, PMF a)
    : obj(o), adr(a)
     {}

  private:
   T    *obj;
   PMF   adr ;

 private:
    // forbid them all due to internal pointers
    const MethodWrapper& operator= (const MethodWrapper&);
    MethodWrapper (const MethodWrapper&);
};


}  // namespace hidden


class Signature;


/** XML RPC Dispatcher (rpc server).
  * The dispacher waits for incoming requests. The request is parsed
  * and the according method is called afterwards the response is
  * converted into xml and sent back to the requester.
  * Direct use of this class is intended for proprietary connections.
  * If you go the usual way via http you ought to use the HttpServer class
  * and run "picoHttpd"
  */
class ULXR_EXPORT Dispatcher
{
 public:
   typedef MethodResponse (*StaticMethodCall_t)(const MethodCall &);


   typedef MethodResponse (*SystemMethodCall_t)(const MethodCall &,
                                                const Dispatcher* disp);

   typedef hidden::MethodWrapperBase*  DynamicMethodCall_t;    // call Wrappers call();

   enum   CallType { CallNone,
                     CallSystem,
                     CallStatic,
                     CallDynamic
                   };

 protected:

   friend StaticMethodCall_t make_method(const StaticMethodCall_t);

   typedef union
   {
      StaticMethodCall_t    static_function;
      SystemMethodCall_t    system_function;
      DynamicMethodCall_t   dynamic_function;
   } MethodCall_t;

   struct ULXR_EXPORT MethodCallDescriptor
   {

     friend class Dispatcher;

   /** Constructs a method call descriptor.
     * @param call  method call data
     */
     MethodCallDescriptor(const MethodCall &call);

   /** Constructs a method call descriptor.
     * @param calltype   type of descriptor
     * @param ret_sig    signature of return type
     * @param name       method name
     * @param signature  signature of parameters
     * @param help       description of method
     */
     MethodCallDescriptor(CallType calltype,
                          const CppString &ret_signature,
                          const CppString &name,
                          const CppString &signature,
                          const CppString &help);

   /** Compares two method call descriptors.
     * @return true: both are NOT equal
     */
     bool operator == (const MethodCallDescriptor &desc) const
       { return getSignature(true, false) == desc.getSignature(true, false); }

   /** Compares two method call descriptors.
     * @return true: both are equal
     */
     bool operator < (const MethodCallDescriptor &desc) const
       { return getSignature(true, false) < desc.getSignature(true, false); }

   /** Gets the signature of the method call.
     * @param with_name    includes also the name of the method (in the middle)
     * @param with_return  includes also the return type (at the begining)
     * @return the signature as string
     */
     CppString getSignature(bool with_name, bool with_return) const;

   /** The the invocation counter.
     * @return how many this methods was invoked
     */
     unsigned long getInvoked() const;

   /** Increments the invocation counter by one
     */
     void incInvoked() const;

   /** Returns the status of availability
     * @return true: method is available
     */
     bool isEnabled() const;

   /** Sets the status of availability
     * @param true: method is available
     */
     void setEnabled(bool ena) const;

   /** Gets the method name
     * @return method name
     */
     CppString getMethodName() const;

   /** Gets the parameter signature
     * @return parameter signature
     */
     CppString getParameterSignature() const;

   /** Gets the return value signature
     * @return return value signature
     */
     CppString getReturnValueSignature() const;

   /** Gets the documentation
     * @return documentation
     */
     CppString getDocumentation() const;

   /** Gets the call type
     * @return call type
     */
     CallType getCallType() const;

    private:

     CallType         calltype;
     CppString        method_name;
     CppString        signature;
     CppString        return_signature;
     CppString        documentation;

     mutable unsigned long  invoked;
     mutable bool           enabled;
   };

   typedef std::map<MethodCallDescriptor, MethodCall_t> MethodCallMap;

   friend void free_dynamic_method(const MethodCallMap::value_type &);

   friend struct MethodCallDescriptor;

 public:

 /** Constructs a dispatcher.
   * @param  conn        pointer to an existing Connection
   * @param  wbxml_mode  true: data is sent as wbxml
   */
   Dispatcher (Protocol* prot = 0, bool wbxml_mode = false);

 /** Destroys the dispatcher.
   * Important: Also frees wrapper objects to worker classes since Dispatcher
   * owns them!
   */
   virtual ~Dispatcher ();

 /** Adds a user defined (static) method to the dispatcher.
   * You access a remote method by sending the "official" name. Sometimes
   * a method accepts different parameter sets (overloading in C++).
   * In this case you add the according signature. Finally you can
   * add a description to show the usage of this method.
   * @param  adr            the pointer to the implementation of the method
   * @param  ret_signature  the signature of the return value
   * @param  name           the name of the method
   * @param  signature      the signature of the parameters
   * @param  help           short usage description
   */
   void addMethod (StaticMethodCall_t adr,
                   const CppString &ret_signature,
                   const CppString &name,
                   const CppString &signature,
                   const CppString &help);

 /** Adds a user defined (dynamic) method to the dispatcher.
   * You access a remote method by sending the "official" name. Sometimes
   * a method accepts different parameter sets (overloading in C++).
   * In this case you add the according signature. Finally you can
   * add a description to show the usage of this method.
   * @param  wrapper        the pointer to the wrapper to the method.
   *                        Important: Dispatcher owns now and deletes this wrapper object!
   * @param  ret_signature  the signature of the return value
   * @param  name           the name of the method
   * @param  signature      the signature of the parameters
   * @param  help           short usage description
   */
   void addMethod (DynamicMethodCall_t wrapper,
                   const CppString &ret_signature,
                   const CppString &name,
                   const CppString &signature,
                   const CppString &help);

 /** Adds a system internal method to the dispatcher.
   * You access a remote method by sending the "official" name. Sometimes
   * a method accepts different parameter sets (overloading in C++).
   * In this case you add the according signature. Finally you can
   * add a description to show the usage of this method.
   * @param  adr            the pointer to the implementation of the method
   * @param  ret_signature  the signature of the return value
   * @param  name           the name of the method
   * @param  signature      the signature of the parameters
   * @param  help           short usage description
   */
   void addMethod (SystemMethodCall_t adr,
                   const CppString &ret_signature,
                   const CppString &name,
                   const CppString &signature,
                   const CppString &help);

 /** Adds a user defined (static) method to the dispatcher.
   * You access a remote method by sending the "official" name. Sometimes
   * a method accepts different parameter sets (overloading in C++).
   * In this case you add the according signature. Finally you can
   * add a description to show the usage of this method.
   * @param  adr            the pointer to the implementation of the method
   * @param  ret_signature  the signature of the return value
   * @param  name           the name of the method
   * @param  signature      the signature of the parameters
   * @param  help           short usage description
   */
   void addMethod (StaticMethodCall_t adr,
                   const Signature &ret_signature,
                   const CppString &name,
                   const Signature &signature,
                   const CppString &help);

 /** Adds a user defined (dynamic) method to the dispatcher.
   * You access a remote method by sending the "official" name. Sometimes
   * a method accepts different parameter sets (overloading in C++).
   * In this case you add the according signature. Finally you can
   * add a description to show the usage of this method.
   * @param  wrapper        the pointer to the wrapper to the method.
   *                        Important: Dispatcher owns now and deletes this wrapper object!
   * @param  ret_signature  the signature of the return value
   * @param  name           the name of the method
   * @param  signature      the signature of the parameters
   * @param  help           short usage description
   */
   void addMethod (DynamicMethodCall_t wrapper,
                   const Signature &ret_signature,
                   const CppString &name,
                   const Signature &signature,
                   const CppString &help);

 /** Adds a system internal method to the dispatcher.
   * You access a remote method by sending the "official" name. Sometimes
   * a method accepts different parameter sets (overloading in C++).
   * In this case you add the according signature. Finally you can
   * add a description to show the usage of this method.
   * @param  adr            the pointer to the implementation of the method
   * @param  ret_signature  the signature of the return value
   * @param  name           the name of the method
   * @param  signature      the signature of the parameters
   * @param  help           short usage description
   */
   void addMethod (SystemMethodCall_t adr,
                   const Signature &ret_signature,
                   const CppString &name,
                   const Signature &signature,
                   const CppString &help);

 /** Waits for an incoming method call.
   * @return the complete call data
   */
   virtual MethodCall waitForCall();

 /** Dispatches the call to the according implementation
   * @param  call  the call data
   * @return the complete response data
   */
   virtual MethodResponse dispatchCall(const MethodCall &call) const;

 /** Sends the response back to the requester.
   * @param  resp        the response data
   */
   virtual void sendResponse(const MethodResponse &resp);

 /** Returns the current connection.
   * @return pointer to the current connection.
   */
   Protocol* getProtocol() const;

 /** Sets the current connection.
   * @param  conn  pointer to the new connection.
   */
   void setProtocol(Protocol*);

 /** Gets the number of available methods.
   * @return number of methods
   */
   unsigned numMethods() const;

 /** Gets method data
   * @param index  index for method
   * @return method data
   */
   const MethodCallDescriptor * const getMethod(unsigned index);

 /** Checks if a method is available
   * @param call  call data
   * @return true if method is available
   */
   bool hasMethod(const MethodCall &call) const;

 protected:

 /** Adds a methode descriptor to the dispatcher.
   * @param  desc   descriptor data
   * @param  mct    pointer to the function
   */
   void addMethodDescriptor (const MethodCallDescriptor &desc,
                             MethodCall_t mct);

 /** Adds internal "system.x" methods.
   */
   void setupSystemMethods();

 /** Lists all methods implemented by this server.
   * @param  calldata  parameters, must be none
   * @param  disp      pointer to actual dispatcher
   * @return Array with names of methods
   */
   static MethodResponse system_listMethods(const MethodCall &calldata,
                                            const Dispatcher *disp);

 /** Returns an array of possible signatures for this method.
   * The signature consists of the return value and all parameters
   * divided by commas.
   * @param  calldata  1 parameter with method name.
   * @param  disp      pointer to actual dispatcher
   * @return Array with names of signatures. non-Array ==> no signature available.
   */
   static MethodResponse system_methodSignature(const MethodCall &calldata,
                                                const Dispatcher *disp);

 /** Returns a documentation string describing the use of this method.
   * If appropriate the return string contains html markup. If there are
   * overloaded methods a description for each may be returned.
   * @param  calldata  1 parameter with method name.
   * @param  disp      pointer to actual dispatcher
   * @returnRpcString with description
   */
   static MethodResponse system_methodHelp(const MethodCall &calldata,
                                           const Dispatcher *disp);

 /** Returns a struct containing the capabilities of this system.
   * Derived class should re-implement addCapabilities() if appropriate
   * and invode parents accordingly.
   * @param  calldata  0 parameters included
   * @param  disp      pointer to actual dispatcher
   * @return Struct with capabilities.
   */
  static MethodResponse system_getCapabilities(const MethodCall &calldata,
                                               const Dispatcher *disp);

 /** Returns a struct containing the capabilities of this system.
   * @param str  reference to a Struct to return the capabilities
   */
   virtual void getCapabilities (Struct &str) const;

 private:

   MethodCallMap             methodcalls;
   Protocol                 *protocol;
   bool                      wbxml_mode;
};


/** Creates a wrapper object to a method of a worker class.
  * new ulxr::hidden::MethodWrapper<classname> (&obj, &classname::method),
  * @param T     reference to worker class
  * @param pmf   adress of member function in worker class
  * @return pointer to wrapper object
  */

#ifndef _MSC_VER

template <class T>
inline hidden::MethodWrapperBase*
make_method(T &w, typename hidden::MethodWrapper<T>::PMF pmf)
{
  return new hidden::MethodWrapper<T> (&w, pmf);
}

#else // work around m$vc bug

template <class T, class U>
inline hidden::MethodWrapper<T>*
make_method(T &w, U pmf)
{
  return new hidden::MethodWrapper<T> (&w, pmf);
}

#endif

/** Specialisation of template to achive uniform look in responder creation.
  * @param ptr   pointer to worker function
  * @return pointer to the function
  */
inline Dispatcher::StaticMethodCall_t
make_method(Dispatcher::StaticMethodCall_t ptr)
{
  return ptr;
}


};  // namespace ulxr


#endif // ULXR_DISPATCHER_H
