/***************************************************************************
               ulxr_tcpip_connection.h  -  tcpip connection

    begin                : Sun Apr 29 2002
    copyright            : (C) 2002-2004 by Ewald Arnold
    email                : ulxmlrpcpp@ewald-arnold.de

    $Id: ulxr_tcpip_connection.h,v 1.22 2004/06/03 16:57:06 ewald-arnold Exp $

 ***************************************************************************/

/**************************************************************************
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#ifndef ULXR_TCPIP_CONNECTION_H
#define ULXR_TCPIP_CONNECTION_H

#ifndef ULXR_OMIT_TCP_STUFF

#include <ulxmlrpcpp.h>  // always first header

#if defined(__WIN32__) || defined(_MSC_VER)
#include <winsock2.h>
#include <windows.h>
typedef int socklen_t;
#endif

#ifdef __unix__
#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/socket.h>
#endif

#include <ulxr_connection.h>


namespace ulxr {


/** Run a generic tcp/ip connection between a client and a server.
  */
class ULXR_EXPORT TcpIpConnection : public Connection
{
 public:

 /** Constructs a generic connection, primarily for a client.
   * The connection is not yet open after construction.
   * @param  server  true: bind socket to adress for server mode
   * @param  domain  domain name of the server
   * @param  port    port on the the server
   */
   TcpIpConnection(bool I_am_server, const CppString &domain, unsigned port = 80);

 /** Constructs a generic connection, primarily for a server.
   * The connection is not yet open after construction.
   * @param  server  true: bind socket to adress for server mode
   * @param  adr     accepted client adress
   * @param  port    port on which the connect is accepted
   */
   TcpIpConnection(bool I_am_server, long adr = INADDR_ANY,  unsigned port = 0);

 /** Destroys the connection.
   */
   virtual ~TcpIpConnection();

 /** Detaches the connection by creating a duplicate of
   * the connection and closing the original connection afterwards.
   * @return the current connection
   */
   virtual Connection *detach();

 /** Opens the connection in client mode.
   */
   virtual void open();

 /** Opens the connection in server mode, thus waiting for
   * connections from clients.
   */
   virtual void accept();

 /** Closes the connection.
   */
   virtual void close();

 /** Gets the host name.
   * @return  the host name.
   */
   CppString getHostName() const
     { return host_name; }

 /** Gets the server doamin.
   * @return  the server domain.
   */
   CppString getServerDomain() const
     { return serverdomain; }

 /** Gets the name of the remote machine.
   * @return  the remote name.
   */
   CppString getPeerName() const
     { return remote_name; }

 /** Converts ascii text to in_addr struct.
   * @param address  literal string of address to convert
   * @param saddr    return struct with data
   */
   void asciiToInAddr(const char *address,
                      struct in_addr &saddr);

 /** Sets abort on close flag for a socket.
   * @param bOn sets SO_LINGER state of the socket on/off.
   * @return status of setsockopt
   * See -s 3SOCKET setsockopt and sys/socket.h for details.
   */
   int abortOnClose(int bOn);

  /** Tests if the current transmission expects a return value.
   * @return true: return value for request
   */
   virtual CppString getInterfaceName();

 /** Portable function to return the current socket error number.
   * @return error number (errno under Unices)
   */
   virtual int getLastError();

 /** Checks if the connection is run as server.
   * @return true: server mode
   */
    bool isServerMode() const;
   
protected:

  class ServerSocketData
  {
    public:
      ServerSocketData(int s_no)
       : count(1),
         socket_no(s_no)
       {}

      int getSocket() const
      {
        return socket_no;
      }

      ~ServerSocketData()
      {
#ifndef __unix__
          ::closesocket(socket_no);
#else
          ::close(socket_no);
#endif
      }

      void incRef()
      {
         ++count;
      }

      int decRef()
      {
        if (count > 0)
          --count;
        return count;
      }

    private:
      int    count;
      int    socket_no;
  };

 /** Sets the pointer to the server-data.
   * @param serv_data   pointer to server
   */
   void setServerData (ServerSocketData *serv_data);

 /** Gets the pointer to the server-data.
   * @return   pointer to server
   */
   ServerSocketData * getServerData () const;

 private:

   CppString           serverdomain;
   unsigned            port;
   ServerSocketData   *server_data;

   CppString           host_name;
   struct sockaddr_in  hostdata;
   socklen_t           hostdata_len;

   CppString           remote_name;
   struct sockaddr_in  remotedata;
   socklen_t           remotedata_len;

 /** Initializes internal variables.
   * @param   port   the port used for the connection
   */
   void init (unsigned port);
};


};  // namespace ulxr

#endif // ULXR_OMIT_TCP_STUFF

#endif // ULXR_TCPIP_CONNECTION_H
